﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Pattern.UnitOfWork
{
    /// <summary>
    /// Universal Repository de diyebiliriz
    /// </summary>
    public class UnitOfWork
        : IUnitOfWork
    {
        private IContextFactory _contextFactory;
        private SampleContext _context;

        public SampleContext Context
        {
            get
            {
                return this._context ?? (_context = _contextFactory.GetContext());
            }
        }

        public UnitOfWork(IContextFactory contextFactory)
        {
            this._contextFactory = contextFactory;
        }

        public UnitOfWork()
        {
            this._contextFactory = new ContextFactory();
        }

        public void Commit()
        {
            this.Context.Commit();
        }

        private bool disposed = false;

        protected virtual void Dispose(bool disposing)
        {
            if (!this.disposed)
            {
                if (disposing)
                {
                    Context.Dispose();
                }
            }
            this.disposed = true;
        }

        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        #region Repositories

        private CategoryRepository _categoryRepository;

        public CategoryRepository CategoryRepository
        {
            get { return this._categoryRepository ?? (_categoryRepository = new CategoryRepository(_contextFactory)); }
        }

        #endregion

    }
}
